#include <QSystemTrayIcon>
#include <QDir>
#include <QtGui>
#include <QApplication>
#include <QFileDialog>
#include <QtDebug>
#include <QFile>
#include <QProcess>
#include <QMenu>
#include <QAction>
#include <QTimer>

#include "wallwindow.h"

//! TODO
//auto update selected folder
//create *.deb or autodeb script
//delete direct from wall by using context menu
//on add file - wrong width of second column
//only one running program
//add settings tab
//icon for icon menu

WallWindow::WallWindow()
{
  ispause = false;

  //Tray menu
  QAction* nextAction = new QAction(tr("&Next"), this);
  connect( nextAction, SIGNAL(triggered()), this, SLOT(nextImage()));

  QAction* previousAction = new QAction(tr("&Previous"), this);
  connect( previousAction, SIGNAL(triggered()), this, SLOT(previousImage()));

  QAction* restoreAction = new QAction(tr("&Restore"), this);
  connect( restoreAction, SIGNAL(triggered()), this, SLOT(showNormal()));

  QAction* quitAction = new QAction(tr("&Quit"), this);
  connect( quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));

  QMenu* trayIconMenu = new QMenu(this);
  trayIconMenu->addAction(nextAction);
  trayIconMenu->addAction(previousAction);
  trayIconMenu->addAction(restoreAction);
  trayIconMenu->addSeparator();
  trayIconMenu->addAction(quitAction);

  //Set tray icon
  trayIcon = new QSystemTrayIcon(this);
  trayIcon->setContextMenu(trayIconMenu);
  trayicon = new QIcon("/usr/share/icons/hicolor/48x48/apps/wallpaper-changer.png");
  trayIcon->setIcon(*trayicon);
  trayIcon->show();

  connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
  this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));

  listTab = new ListTab;
  settTab = new settingTab;
  tabWidget = new QTabWidget;
  tabWidget->addTab(listTab, tr("Images"));
  tabWidget->addTab(settTab, tr("Folders"));

  setMinimumHeight(300);
  resize(400,500);
  setCentralWidget(tabWidget);
  setWindowIcon(*trayicon);
}

void WallWindow::nextImage()
{
  listTab->setImage();
}

void WallWindow::previousImage()
{
  listTab->currentPosition-=2;
  listTab->setImage();
}

void WallWindow::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
  if (reason==QSystemTrayIcon::Trigger)
  {
    qDebug()<<"Fast swich";
    listTab->setImage();
  }
  if (reason==QSystemTrayIcon::MiddleClick)
  {
    if (ispause==true)
    {
      listTab->startButton_clicked();
      qDebug()<<"Scrolling started";
      ispause=false;
    }
    else
    {
      listTab->startButton->setText("Start");
      listTab->switchTimer->stop();
      ispause=true;
      trayIcon->setToolTip("Scrolling stopped");
      qDebug()<<"Scrolling stopped";
    }
  }
}

ListTab::ListTab(QWidget *parent) : QWidget(parent)
{
  delIcon = new QIcon("/usr/share/icons/gnome/48x48/actions/edit-delete.png");
  settings = new QSettings(QSettings::IniFormat, QSettings::UserScope,
                           "wallpapers-changer", "wallchanger");
  switchTimer = new QTimer;
  connect(switchTimer,SIGNAL(timeout()),this,SLOT(setImage()));

  switch_time = 60;
  row_num = 0;
  number = 0;
  currentPosition = 0;

  //Table configure
  table = new QTableWidget;
  table->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
  table->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
  table->setAutoScroll(false);
  table->setEditTriggers(QAbstractItemView::NoEditTriggers);
  table->setSelectionMode(QAbstractItemView::NoSelection);
  //table->horizontalHeader()->setResizeMode(2,QHeaderView::ResizeToContents);
  table->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
  table->setShowGrid(false);
  table->horizontalHeader()->setVisible(false);
  table->verticalHeader()->setVisible(false);

  imgpos_label = new QLabel("Image position:");
  countlabel = new QLabel("Image count:");
  scroll_label = new QLabel("Scroll time(minutes):");
  scroll_label->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);

  startButton = new QPushButton(tr("Start"));
  //quitButton = new QPushButton(tr("Quit"));
  clearButton = new QPushButton(tr("Clear"));
  addButton = new QPushButton(tr("Add"));

  QStringList posList, img_order_list;
  posList<<"None"<<"Wallpaper"<<"Centered"<<"Scaled"
         <<"Stretched"<<"Zoom"<<"Spanned";
  img_order_list<<"Linear"<<"Shuffle";

  timeBox = new QSpinBox;
  pos_comboBox = new QComboBox;
  pos_comboBox->addItems(posList);
  onlynames_checkBox = new QCheckBox(tr("Only names"));
  img_order_label = new QLabel(tr("Order of showing image:"));
  order_comboBox = new QComboBox;
  order_comboBox->addItems(img_order_list);

  QGridLayout *layout = new QGridLayout;
  layout->setMargin(1);

  layout->addWidget(scroll_label,0,0);
  layout->addWidget(timeBox,0,6,Qt::AlignRight);

  layout->addWidget(countlabel,1,0);
  layout->addWidget(onlynames_checkBox,1,6,Qt::AlignRight);

  layout->addWidget(table,2,0,1,0);

  layout->addWidget(imgpos_label,3,0);
  layout->addWidget(pos_comboBox,3,6,Qt::AlignRight);

  layout->addWidget(img_order_label,4,0,1,0);
  layout->addWidget(order_comboBox,4,6,Qt::AlignRight);

  layout->addWidget(clearButton,5,0);
  layout->addWidget(addButton,5,1);
  layout->addWidget(startButton,5,6);
  //layout->addWidget(quitButton,5,6);
  setLayout(layout);

  if (settings->value("settings/show_only_names")==true)
      onlynames_checkBox->setCheckState(Qt::Checked);
  timeBox->setValue(settings->value("settings/scroll_time",10).toInt());
  scroll_label->setText(settings->value("settings/in", "Scroll time(minutes):").toString());
  pos_comboBox->setCurrentIndex(settings->value("settings/img_position",0).toInt());
  order_comboBox->setCurrentIndex(settings->value("settings/img_order",0).toInt());
  currentPosition = settings->value("settings/currentPosition",0).toInt();

  QStringList tempList;
  tempList = settings->value("settings/random_number_list").toStringList();
  for (int i=0; i<tempList.count(); i++)
      random_number_list.append(tempList.at(i).toInt());

  if (scroll_label->text().contains("hour")) {
      timeBox->setMaximum(24);
      timeBox->setMinimum(0);
  }

  //connect(clearButton,SIGNAL(clicked()),this,SLOT(quitButton_clicked()));
  connect(startButton, SIGNAL(clicked()), this, SLOT(startButton_clicked()));
  connect(clearButton, SIGNAL(clicked()), this, SLOT(clearButton_clicked()));
  connect(addButton  , SIGNAL(clicked()), this, SLOT(addButton_clicked())  );

  connect(onlynames_checkBox,SIGNAL(stateChanged(int)),this,
          SLOT(on_onlynames_checkBox_stateChanged(int)));

  connect(timeBox,SIGNAL(valueChanged(int)),this,
          SLOT(on_timeBox_valueChanged(int)));

  connect(pos_comboBox,SIGNAL(currentIndexChanged(QString)),this,
          SLOT(on_pos_comboBox_currentIndexChanged(QString)));

  connect(order_comboBox,SIGNAL(currentIndexChanged(QString)),this,
          SLOT(order_comboBox_currentIndexChanged(QString)));

  connect(table,SIGNAL(cellClicked(int,int)),this,
          SLOT(on_table_cellClicked(int,int)));

  connect(table,SIGNAL(cellDoubleClicked(int,int)),this,
          SLOT(on_table_cellDoubleClicked(int,int)));

  allFiles=load_and_show();
  if (random_number_list.isEmpty()) random_number_list=createRandomNumberList();
  createRandomNumberList();
  if (allFiles.count()!=0) startButton_clicked();
}

QFileInfoList ListTab::load_and_show()
{
  if (!settings->value("settings/dirList").toStringList().isEmpty()) {
    qDebug()<<"var";
    QStringList dirsList;
    dirsList = settings->value("settings/dirList").toStringList();
    qDebug()<<"var";
    QDir tempDir;
    QStringList fileFilter;
    fileFilter<<"*.png"<<"*.jpg"<<"*.jpeg"<<"*.bmp"<<"*.tiff"<<"*.tif";
    for (int i = 0; i<dirsList.count(); ++i) {
      tempDir.setPath(dirsList.at(i));
      allFiles.append(tempDir.entryInfoList(fileFilter));
    }

  } else {
      QStringList files = settings->value("settings/additionalImagesList").toStringList();
      for (int i=0; i<files.count(); i++) {
        QFileInfo info(files[i]);
        if (info.exists())
          allFiles.append(info);
      }
  }
  tableFill();
  return allFiles;
}

void ListTab::tableFill()
{
  table->setRowCount(allFiles.count());
  table->setColumnCount(2);
  table->setColumnWidth(1,500); //bug
  for(int i=0; i<allFiles.count(); i++)
  {
    QTableWidgetItem* iconItem = new QTableWidgetItem();
    iconItem->setIcon(*delIcon);
    table->setItem(i, 0, iconItem);

    QTableWidgetItem* textItem = new QTableWidgetItem();
    if (onlynames_checkBox->isChecked())
        textItem->setText(QString(allFiles.at(i).fileName()));
    else
        textItem->setText(QString(allFiles.at(i).absoluteFilePath()));
    table->setItem(i, 1, textItem);
  }
  countlabel->setText("Images count: "+QString::number(allFiles.count()));
  table->resizeColumnsToContents();
}

int ListTab::setImage()
{
  if (row_num!=0)
    number=row_num;

  if (order_comboBox->currentText()=="Linear" && row_num==0) {
    number = linearList.at(currentPosition);

    qDebug()<<"Set image:"<<allFiles.at(number).absoluteFilePath();
    processStart("/desktop/gnome/background/picture_filename",
                 allFiles.at(number).absoluteFilePath());
    currentPosition = number+1;
  } else if (order_comboBox->currentText()=="Shuffle") {
    number=random_number_list.at(currentPosition);
    qDebug()<<"Set image:"<<allFiles.at(number).absoluteFilePath();
    processStart("/desktop/gnome/background/picture_filename",
                 allFiles.at(number).absoluteFilePath());
    if (row_num==0) currentPosition++;
  }

  table->scrollToItem(table->item(number,1),QAbstractItemView::PositionAtTop);
  table->horizontalScrollBar()->setValue(0);

  //trayIcon->setToolTip("Now: "+allFiles.at(number).fileName());

  if (currentPosition==random_number_list.count()) {
    random_number_list=createRandomNumberList();
    currentPosition=0;
  }
  if (currentPosition==linearList.count()) {
    createRandomNumberList();
    currentPosition=0;
  }

  row_num=0;
  config_save();
  return 0;
}

void ListTab::processStart(QString path, QString arg)
{
  QProcess myProcess;
  QString program = "gconftool-2";
  QStringList arguments;
  arguments<<"--type"<<"string"<<"--set"<<path<<arg;
  myProcess.start(program, arguments);
  myProcess.waitForFinished();
}

QList<int> ListTab::createRandomNumberList()
{
  QList<int> temp1, temp2;
  int rand_num;
  qsrand(QDateTime::currentDateTime().toTime_t());

  for (int i=0; i<allFiles.count(); ++i) temp1<<i;
  linearList=temp1;

  for (int i=0; i<allFiles.count(); ++i) {
    rand_num=qrand()%temp1.count();
    temp2<<temp1.at(rand_num);
    temp1.removeAt(rand_num);
  }
  return temp2;
}

void ListTab::startButton_clicked()
{
  if (startButton->text()=="Stop") {
    switchTimer->stop();
    qDebug()<<"Scrolling stopped";
    startButton->setText("Start");
  } else {
    if (scroll_label->text().contains("minutes")) switch_time=timeBox->text().toInt()*60*1000;
    else switch_time=timeBox->text().toInt()*60*60*1000;
    //switch_time=1000; //for debug
    switchTimer->start(switch_time);
    if (switchTimer->isActive()) qDebug()<<"Scrolling started";
    startButton->setText("Stop");
    setImage();
  }
  config_save();
}

void ListTab::on_onlynames_checkBox_stateChanged(int x)
{
  for(int i=0; i<allFiles.count(); i++) {
    QTableWidgetItem* Item = new QTableWidgetItem();
    if (x==2) Item->setText(QString(allFiles.at(i).fileName()));
    if (x==0) Item->setText(QString(allFiles.at(i).absoluteFilePath()));
    table->setItem(i, 1, Item);
  }
  if (onlynames_checkBox->isChecked())
    settings->setValue("settings/show_only_names", true);
  else
    settings->setValue("settings/show_only_names", false);
}

void ListTab::on_timeBox_valueChanged(int time)
{
  if (time>59 && scroll_label->text().contains("minutes")) {
    timeBox->setValue(1);
    timeBox->setMaximum(24);
    timeBox->setMinimum(0);
    scroll_label->setText("Scroll time(hour):");
  }
  if (time==0) {
    timeBox->setMaximum(60);
    timeBox->setMinimum(1);
    timeBox->setValue(59);
    scroll_label->setText("Scroll time(minutes):");
  }
  settings->setValue("settings/scroll_time", timeBox->value());
}

void ListTab::on_table_cellClicked(int row, int column)
{
  if (column==0) {
    table->removeRow(row);
    allFiles.removeAt(row);
    config_save();
  }
}

void ListTab::on_table_cellDoubleClicked(int row, int column)
{
  row_num=row;
  if (column==1) setImage();
}

void ListTab::on_pos_comboBox_currentIndexChanged(QString name)
{
  processStart("/desktop/gnome/background/picture_options",name.toLower());
}

void ListTab::order_comboBox_currentIndexChanged(QString)
{
  currentPosition = number + 1;
}

void ListTab::config_save()
{
  if (scroll_label->text().contains("minutes"))
    settings->setValue("settings/in", "Scroll time(minutes):");
  else
    settings->setValue("settings/in", "Scroll time(hour):");

  settings->setValue("settings/img_position", pos_comboBox->currentIndex());

  settings->setValue("settings/img_order", order_comboBox->currentIndex());

  settings->setValue("settings/currentPosition", currentPosition);

  QStringList tempList;
  for (int i=0; i<random_number_list.count(); i++)
    tempList.append(QString::number(random_number_list.at(i)));
  settings->setValue("settings/random_number_list", tempList);

  tempList.clear();
  for (int i=0; i<additionalImages.count(); i++)
    tempList.append(additionalImages.at(i).absoluteFilePath());
  settings->setValue("settings/additionalImagesList", tempList);
}

void ListTab::addButton_clicked()
{
  QStringList added_files = QFileDialog::getOpenFileNames(this,
                                         "Open Image(s)",
                                         QDir::homePath (),
                                         "Image Files (*.png *.jpg *.bmp *.tiff *.tif)");

  for (int i=0; i<added_files.count(); i++) {
    QFileInfo info(added_files[i]);
    additionalImages.append(info);
  }
  allFiles.append(additionalImages);

  tableFill();
  random_number_list=createRandomNumberList();
  createRandomNumberList();
  currentPosition=0;
  config_save();
}

void ListTab::clearButton_clicked()
{
  allFiles.clear();
  table->clear();
  table->setColumnWidth(1,500);
  config_save();
}

void WallWindow::hideEvent(QHideEvent * event)
{
  hide();
  event->ignore();
  listTab->config_save();
}

void WallWindow::closeEvent(QCloseEvent *event)
{
  event->ignore();
  hide();
}

settingTab::settingTab(QWidget *parent) : QWidget(parent)
{
  settings = new QSettings(QSettings::IniFormat, QSettings::UserScope,
                           "wallpapers-changer", "wallchanger");

  picFolders = new QLabel("Pictures folders:",this);
  addFolderButton = new QPushButton("Add folder",this);
  connect(addFolderButton, SIGNAL(clicked()), this, SLOT(on_addFolderButton_clicked()));
  folderListWidget = new QListWidget(this);
  QGridLayout *my = new QGridLayout;

  //QStringList dirList;
  //dirList = new QStringList;
  dirList = settings->value("settings/dirList").toStringList();
  //dirList<<"/media/data/Additional/Pictures"<<"/media/data/Additional/Images";

  my->addWidget(picFolders,0,0,1,3);
  my->addWidget(folderListWidget,1,0,1,3);
  my->addWidget(addFolderButton,2,0);
  setLayout(my);

  for (int i=0; i<dirList.count(); ++i) {
    folderListWidget->addItem(dirList.at(i));
  }

  if (!dirList.isEmpty())
    settings->setValue("settings/dirList",dirList);
}

void settingTab::on_addFolderButton_clicked()
{
  QString selectedDir = QFileDialog::getExistingDirectory(this,
                                                          "Open Image(s)",
                                                          "/media/data",
                                                          QFileDialog::ShowDirsOnly);
  //QDir pictureDirs;
  //pictureDirs->setPath(selectedDir.toAscii());
  folderListWidget->addItem(selectedDir);
  dirList.append(selectedDir);

  qDebug()<<dirList;

  if (!dirList.isEmpty())
    settings->setValue("settings/dirList",dirList);

  //ListTab my;
  if (!settings->value("settings/dirList").toStringList().isEmpty()) {
    if (m_pListTab)
    {
         //m_pListTab->load_and_show();
    }
  }
  //ListTab *temp;
  //temp->load_and_show();
}
